import logging

from bluedart_exceptions import ParamValidationError
from zeep import Client
from zeep.exceptions import ValidationError, Fault
from response_handler import ResponseHandler


class BlueDart(object):
    def __init__(self, request_url, https=False):
        self.Profile = None
        self.response = {}
        self.logger = logging.getLogger(__name__)
        self.client = Client(request_url)
        self._prepare_pre_request_data()

    def _set_profile_credentials(self, user_profile):
        """
        Set profile credentials for the wsdl request.
        :param user_profile: Contains Profile object of creds.
        :return: WSDL profile param.
        """
        return self.Profile(
            Api_type=user_profile.api_type,
            LoginID=user_profile.login_id,
            LicenceKey=user_profile.license_key,
            Version=user_profile.version,
            Customercode=user_profile.customer_code
        )

    def _prepare_pre_request_data(self):

        """
        This should be treated as a abstract method and to be over ridden by each subclass.
        Prepare the wsdl request prefixes.
        :return: None
        """
        pass

    def _send_request(self):
        """
        This is abstract method and have to be over ridden in subclass.
        Inside subclass it will decide which method to call for WSDL client.
        :return: None
        """
        pass

    def send_request(self):
        """
        Used to send the final SOAP request and to be called by subclass.
        It internally override the _send_request of subclass.
        :return: None
        """
        try:
            response = self._send_request()
            self.response = ResponseHandler(response)

        except Fault as fault:
            raise ParamValidationError(message=fault.message, code=fault.code)

        except Exception as exc:
            pass
        return self.response



"""
File: user_profile
Author: Vineet Dubey <vineetdubey@gofynd.com>
Date: 10/02/2017

This is the user credentials  for Ecomm profile.
"""


class UserProfile(object):
    """
    Set the user credentials for API requests.
    """
    def __init__(self, client_name, api_token, debug=False):
        self.client_name = client_name
        self.api_token = api_token
        self.debug = debug



"""
File: user_profile
Author: Om Prakash <omprakash@gofynd.com>
Date: 10/02/2017

This is the user creds for bluedart profile.
"""


class UserProfile(object):
    """
    Set the user credentials for API requests.
    """
    def __init__(self, api_type, login_id, licence_key, area=None, customer_code=None, is_admin=None, password=None,
                 version=None, tracking_license_key=None, debug=False):
        self.api_type = api_type
        self.login_id = login_id
        self.password = password
        self.area = area
        self.customer_code = customer_code
        self.is_admin = is_admin
        self.license_key = licence_key
        self.version = version
        self.tracking_license_key = tracking_license_key
        self.debug = debug



"""
File: services
Author: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 04/03/2017

It contains XML helper parser methods required to parse the response sent by delhivery.
"""
import xml.etree.cElementTree as ET


class DelhiveryXMLHelper(object):
    def __init__(self, xml_string):
        self.xml_string = xml_string
        self.status_dict = dict()

    def parse(self):
        if self.xml_string:
            root = ET.fromstring(self.xml_string)
            for child in root:
                self.status_dict[child.tag] = child.text
        return self.status_dict



"""
File: response_handler
Author: Om Prakash <omprakash@gofynd.com>
Date: 17/02/2017

It handles the response in bluedart.
"""


class ResponseHandler(object):

    def __init__(self, response):
        self.status_code = 200
        self.response = response
        self.response_attributes = ['AWBNo', 'CCRCRDREF', 'DestinationArea', 'DestinationLocation', 'IsError',
                                    'ShipmentPickupDate', 'TokenNumber', "Status"]
        self.set_properties()

    def set_properties(self):
        for attr in self.response_attributes:
            if attr not in ['IsError']:
                    self.response.__setattr__(attr, str(self.response[attr]))
            else:
                self.response.__setattr__(attr, self.response[attr])
        return self

    def json(self):
        json_dict = {}
        for attr in self.response_attributes:
            json_dict[attr] = self.response[attr]

        return json_dict



"""
File: constants
Author: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 04/03/2017

This file contains the constants needed for making Ecomm API calls
"""

ECOM_PLACE_SHIPMENT = "/apiv2/manifest_awb/"
ECOM_CANCEL_SHIPMENT = "/apiv2/cancel_awb/"
# Ecomm
ECOM_DEBUG_BASE_URL = "http://ecomm.prtouch.com"
ECOM_BASE_URL = "http://api.ecomexpress.in"
TEST_CREDS = {
    "username": 'ecomexpress',
    "password": 'K782&%#jkhas',
    "debug": True
}


"""
File: constants
Author: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 04/03/2017

This file contains the constants needed for making Ecomm API calls
"""

DELHIVERY_CREATE_PACKAGE = "/cmu/push/json/?token={}"  # .format(DELHIVERY_API_TOKEN)
DELHIVERY_CREATE_PACKAGE_HEADERS = {"content-type": "application/x-www-form-urlencoded"}
DELHIVERY_CREATE_PICKUP_HEADERS = {'Content-Type': 'application/json',
                                   'Authorization': 'Token {}'}  # .format(DELHIVERY_API_TOKEN)}
DELHIVERY_CANCEL_PACKAGE_HEADERS = DELHIVERY_CREATE_PICKUP_HEADERS
DELHIVERY_CANCEL_PACKAGE = "/api/p/edit"
DELHIVERY_CREATE_PICKUP = "/fm/request/new/"
TEST_CREDS = {
    'client_name': "XXX - TEST",
    'api_token': '98723ksdjwiou923709xns980932nsdfie0'
}
DELHIVERY_DEBUG_BASE_URL = "http://test.delhivery.com"
DELHIVERY_BASE_URL = "https://track.delhivery.com"
